#!/usr/bin/env python

from gnuradio import gr
from gnuradio import blocks, analog, filter
import scipy, pylab
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from scipy import fftpack

# Create a low pass filter
firdes_taps = filter.firdes.low_pass_2(1, 1, 0.2, 0.1, 60)
optfir_taps = filter.optfir.low_pass(1, 1, 0.13, 0.2675, 0.01, 60)

# Generate white noise
src = analog.noise_source_c(analog.GR_GAUSSIAN, 1)

# Limit the number of samples to 1 million
N = 1000000
head = blocks.head(gr.sizeof_gr_complex, N)

# Create two filters with the same taps
firdes_filt = filter.fft_filter_ccc(1, firdes_taps)
optfir_filt = filter.fft_filter_ccc(1, optfir_taps)

# Sinks to store the data
snk0 = blocks.vector_sink_c()
snk1 = blocks.vector_sink_c()

# Create the top block and connect the blocks
tb = gr.top_block()
tb.connect(src, head)
tb.connect(head, firdes_filt, snk0)
tb.connect(head, optfir_filt, snk1)
tb.run()

data0 = scipy.array(snk0.data())
data1 = scipy.array(snk1.data())

time = scipy.arange(0, N, 1)
freq = scipy.arange(-1.0/2.0, 1.0/2.0, 1.0/N)
X0 = 20*scipy.log10(abs(fftpack.fftshift(fftpack.fft(data0, N))))
X1 = 20*scipy.log10(abs(fftpack.fftshift(fftpack.fft(data1, N))))

#fig = pylab.figure(1, figsize=(12,10), facecolor='w')
#sp1 = fig.add_subplot(2,1,1)
#sp1.plot(time[0:200], data0[0:200], 'b', label="firdes", linewidth=4)
#sp1.plot(time[0:200], data1[0:200], 'r', label="optfir", linewidth=2)
#sp1.set_xlabel("Time (samples)", fontsize=16, fontweight='bold')
#sp1.set_ylabel("Amplitude", fontsize=16, fontweight='bold')
#sp1.grid()
#sp1.legend()
#
#sp2 = fig.add_subplot(2,1,2)
#sp2.plot(freq, X0, 'b', label="firdes")
#sp2.plot(freq, X1, 'r', label="optfir", alpha=0.5)
#sp2.set_xlabel("Normalized Frequency", fontsize=16, fontweight='bold')
#sp2.set_ylabel("Magnitude (dB)", fontsize=16, fontweight='bold')
#sp2.grid()
#sp2.legend()


X0 = 20*scipy.log10(abs(fftpack.fftshift(fftpack.fft(firdes_taps, N))))
X1 = 20*scipy.log10(abs(fftpack.fftshift(fftpack.fft(optfir_taps, N))))

fig2 = pylab.figure(2, figsize=(12,10), facecolor='w')
ax1 = fig2.add_subplot(1,1,1)
ax1.plot(freq, X0, 'b', label="firdes", linewidth=2)
ax1.plot(freq, X1, 'r', label="optfir", linewidth=2, alpha=0.5)
ax1.set_xlabel("Normalized Frequency", fontsize=16, fontweight='bold')
ax1.set_ylabel("Magnitude (dB)", fontsize=16, fontweight='bold')
ax1.grid()
ax1.legend()
ax1.set_xlim([0, 1.0/2.0])
ax1.set_ylim([-200, 10])

ax2 = inset_axes(ax1, width=3.5, height=3.5, loc=3,
                 bbox_to_anchor=(0.175, 0.15), 
                 bbox_transform=ax1.figure.transFigure)

#ax2 = pylab.subplot2grid((3,3), (2,0), colspan=1, rowspan=1)
ax2.plot(freq, X0, 'b', label="firdes", linewidth=2)
ax2.plot(freq, X1, 'r', label="optfir", linewidth=2, alpha=0.5)
ax2.set_xlim([0, 0.15])
ax2.set_ylim([-0.03, 0.03])


pylab.show()
